/*
 * gestures.js
 * gesture listener
 * c. 2011 Richard M. Salter, Logical Labs. All rights reserved
 */

exports.GestureListener = function(opengl, pcallback, tcallback, rcallback) {

	var rect = opengl.bounds;
	var width = rect.width;
	var height = rect.height;
	var scaleStart = 1;
	var pinchStart = 0;
	var newPinch = true;
	var pinching = false;
	var lastRot = 0;
	var newRot = true;
	var rotating = false;
	var xstart = 0;
	var ystart = 0
	var deltaX = undefined; 
	var deltaY = undefined; 
	var xpos = 0;
	var ypos = 0;
	var xcur = 0; 
	var ycur = 0;
	var scale = 1;
	var rotation = 0;
	
	var me = this;
	opengl.addEventListener('pinch', function(e) {
		if (newPinch) {
			newPinch = false;
			scaleStart = scale;
			pinchStart = e.scale;
		} else {
			scale = Math.max(scaleStart + e.scale - pinchStart, .001);
		}
		pinching = true;
		if (pcallback != null) pcallback(scale);
	});

	opengl.addEventListener('rotation', function(e) {
		if (newRot) {
			newRot = false;
			lastRot = e.rotation;
		} else {
			rotation = e.rotation - lastRot;
			lastRot = e.rotation;
		}
		rotating = true;
		if (rcallback != null) rcallback(rotation);
	});

	opengl.addEventListener('touchstart', function(e) {
		xstart = e.x;
		ystart = e.y;
		newPinch = true;
		newRot = true;
	});
	
	opengl.addEventListener('touchmove', function(e) {
		if (pinching) return;
		if (deltaX && deltaY) {
			pinching = false;
			deltaX = e.x - deltaX;
			deltaY = deltaY - e.y;
			if (tcallback != null && !pinching) tcallback(deltaX, deltaY);
		}
		xpos = 2*(e.x - xstart)/width;
		ypos = 2*(e.y - ystart)/height;
		deltaX = e.x;
		deltaY = e.y;
	});
	
	opengl.addEventListener('touchend', function(e) {
		deltaX = undefined;
		deltaY = undefined;
		xcur = xcur-xpos;
		ycur = ypos+ycur;
		xpos = 0;
		ypos = 0;
		pinching = false;
		newPinch = true;
	});

	this.xmove = function() {
		return xpos-xcur;
	}
	
	this.ymove = function() {
		return -(ypos+ycur);
	}

	this.scale = function() {
		return scale;
	}
}
