/* helpers.js
 *
 * helper functions for using opengl
 * 
 * Developed for Appcelerator by:
 *   Richard M. Salter
 *   Logical Labs, LLC
 * 
 * Please direct all questions, feedback, and concerns to info@appcelerator.com.
 *
 * Copyright (c) 2011 by Appcelerator, Inc. All Rights Reserved.
 *  
 */

exports.TiOpenGLUtil = {
	startAnimation : function(view, callback, interval, delay, target) {
		if (delay == null || target == null) {
			return setInterval(function(e){callback(view, new Date().getTime());}, interval);
		};
		if (delay > 0) {
			setTimeout(
				function() {
					target.timer = setInterval(function(e){callback(view, new Date().getTime());}, interval);
				}, 
				delay
			);
		}
	},
	addComponents : function(parent, children) {
		for (var idx in children) {
			if (!children[idx].noadd) parent.add(children[idx]);
		}
	},
	makeBuffers : function(view, dataBufs) {
		var bufs0 = [dataBufs.vertexDB, 
					dataBufs.colorDB, 
					dataBufs.normalDB,
					dataBufs.textureDB];
		var idb = dataBufs.indexDB;					
		bufs = [];
		starts = [];
		tot = 0;
		for (idx in bufs0) {
			if (bufs0[idx] != null) {
				bufs.push(bufs0[idx]);
				starts.push(tot);
				tot += bufs0[idx].size;
			}
		}
		var vbo = view.glGenBuffers((idb == null) ? 1 : 2);
		view.glBindBuffer(view.GL_ARRAY_BUFFER, vbo[0]);
		view.mapBuffer(view.GL_ARRAY_BUFFER, view.GL_STATIC_DRAW, bufs);
		if (idb != null) {
			view.glBindBuffer(view.GL_ELEMENT_ARRAY_BUFFER, vbo[1]);
			view.glBufferData(view.GL_ELEMENT_ARRAY_BUFFER, idb.getSize(), idb, view.GL_STATIC_DRAW);
		}
		return {
			vbo : vbo, 
			start : starts
		}
	},
	starter : function(n_, f_) {
		this.n = n_
		this.f = f_
		var me = this;
		this.wait = function(e) {
			me.n--;
			if (me.n == 0) {me.f();}
		}
	},
	
	// www.sean.co.uk 
	// (yuck)
	
	 pauseComp : function(millis) 
	{
		var date = new Date();
		var curDate = null;
	
		do { curDate = new Date(); } 
		while(curDate-date < millis);
	} 
	
}

exports.defaultInit = {
	setup: function(view) {
		view.setCurrentContext();
    	view.glMatrixMode(view.GL_PROJECTION);
	    view.glLoadIdentity();
	    var frame = view.bounds;
	   	var scale = view.contentScaleFactor;
    // Setup the view port in Pixels
	    view.glOrthof(0, frame.width * scale, 0, frame.height * scale, -1, 1);
	    view.glViewport(0, 0, frame.width * scale, frame.height * scale);
	    view.glMatrixMode(view.GL_MODELVIEW);
	    view.glLoadIdentity();    
	    view.glDisable(view.GL_DITHER);
	    view.glEnable(view.GL_TEXTURE_2D);
	    view.glEnableClientState(view.GL_VERTEX_ARRAY);
	    view.glEnable(view.GL_BLEND);
    // Set a blending function appropriate for premultiplied alpha pixel data
	    view.glBlendFunc(view.GL_ONE, view.GL_ONE_MINUS_SRC_ALPHA);
	    view.glEnable(view.GL_POINT_SPRITE_OES);
	    view.glTexEnvf(view.GL_POINT_SPRITE_OES, view.GL_COORD_REPLACE_OES, view.GL_TRUE); 		
	}
}

exports.getOrientation = function(o)
{
	switch (o)
	{
		case Titanium.UI.PORTRAIT:
		{
			return 'portrait';
		}
		case Titanium.UI.UPSIDE_PORTRAIT:
		{
			return 'upside portrait';
		}
		case Titanium.UI.LANDSCAPE_LEFT:
		{
			return 'landscape left';
		}
		case Titanium.UI.LANDSCAPE_RIGHT:
		{
			return 'landscape right';
		}
		case Titanium.UI.FACE_UP:
		{
			return 'face up';
		}
		case Titanium.UI.FACE_DOWN:
		{
			return 'face down';
		}
		case Titanium.UI.UNKNOWN:
		{
			return 'unknown';
		}
	}
}

exports.isPortrait = function(o) {
	return o == Ti.UI.PORTRAIT || o == Ti.UI.UPSIDE_PORTRAIT;
}

exports.isLandscape = function(o) {
	return o == Ti.UI.LANDSCAPE_LEFT || o == Ti.UI.LANDSCAPE_RIGHT;
};

if (!Array.prototype.map)
{
  Array.prototype.map = function(fun /*, thisp*/)
  {
    var len = this.length;
    if (typeof fun != "function")
      throw new TypeError();

    var res = new Array(len);
    var thisp = arguments[1];
    for (var i = 0; i < len; i++)
    {
      if (i in this)
        res[i] = fun.call(thisp, this[i], i, this);
    }

    return res;
  };
}