/* matrix.js
 * matrix utility functions
 *
 * Adapted to Javascript from http://iphonedevelopment.blogspot.com/2009/04/opengl-es-from-ground-up-part-7.html
 *
 * Author: Richard M. Salter, Logical Labs
 *
 * Copyright 2011, Logical Labs. All rights reserved
 */

exports.Matrix = (function () {
	var obj = {
		m3DMultiply: function(m1, m2) {
			var result = [];
			result[0] = m1[0] * m2[0] + m1[4] * m2[1] + m1[8] * m2[2] + m1[12] * m2[3];
			result[1] = m1[1] * m2[0] + m1[5] * m2[1] + m1[9] * m2[2] + m1[13] * m2[3];
			result[2] = m1[2] * m2[0] + m1[6] * m2[1] + m1[10] * m2[2] + m1[14] * m2[3];
			result[3] = m1[3] * m2[0] + m1[7] * m2[1] + m1[11] * m2[2] + m1[15] * m2[3];

			result[4] = m1[0] * m2[4] + m1[4] * m2[5] + m1[8] * m2[6] + m1[12] * m2[7];
			result[5] = m1[1] * m2[4] + m1[5] * m2[5] + m1[9] * m2[6] + m1[13] * m2[7];
			result[6] = m1[2] * m2[4] + m1[6] * m2[5] + m1[10] * m2[6] + m1[14] * m2[7];
			result[7] = m1[3] * m2[4] + m1[7] * m2[5] + m1[11] * m2[6] + m1[15] * m2[7];

			result[8] = m1[0] * m2[8] + m1[4] * m2[9] + m1[8] * m2[10] + m1[12] * m2[11];
			result[9] = m1[1] * m2[8] + m1[5] * m2[9] + m1[9] * m2[10] + m1[13] * m2[11];
			result[10] = m1[2] * m2[8] + m1[6] * m2[9] + m1[10] * m2[10] + m1[14] * m2[11];
			result[11] = m1[3] * m2[8] + m1[7] * m2[9] + m1[11] * m2[10] + m1[15] * m2[11];

			result[12] = m1[0] * m2[12] + m1[4] * m2[13] + m1[8] * m2[14] + m1[12] * m2[15];
			result[13] = m1[1] * m2[12] + m1[5] * m2[13] + m1[9] * m2[14] + m1[13] * m2[15];
			result[14] = m1[2] * m2[12] + m1[6] * m2[13] + m1[10] * m2[14] + m1[14] * m2[15];
			result[15] = m1[3] * m2[12] + m1[7] * m2[13] + m1[11] * m2[14] + m1[15] * m2[15];
			return result;
		},
		m3DSetIdentity: function () {
			var matrix = [];
			matrix[0] = matrix[5] =  matrix[10] = matrix[15] = 1.0;
			matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0;
			matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0;
			matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;
			return matrix;
		},
		m3DSetTranslation: function(xTranslate, yTranslate, zTranslate) {
			var matrix = [];
			matrix[0] = matrix[5] =  matrix[10] = matrix[15] = 1.0;
			matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0;
			matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0;
			matrix[11] = 0.0;
			matrix[12] = xTranslate;
			matrix[13] = yTranslate;
			matrix[14] = zTranslate;
			return matrix;
		},
		m3DSetScaling: function(xScale, yScale, zScale) {
			var matrix = [];			
			matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0;
			matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0;
			matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;
			matrix[0] = xScale;
			matrix[5] = yScale;
			matrix[10] = zScale;
			matrix[15] = 1.0;
			return matrix;
		},
		m3DSetUniformScaling: function(scale) {
			return obj.m3DSetScaling(matrix, scale, scale, scale);
		},
		m3DSetXRotationUsingRadians: function(radians) {
			var matrix = [];			
			matrix[0] = matrix[15] = 1.0;
			matrix[1] = matrix[2] = matrix[3] = matrix[4] = 0.0;
			matrix[7] = matrix[8] = 0.0;
			matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;

			matrix[5] = Math.cos(radians);
			matrix[6] = -Math.sin(radians);
			matrix[9] = -matrix[6];
			matrix[10] = matrix[5];
			return matrix;
		},
		m3DSetXRotationUsingDegrees: function(degrees) {
			return obj.m3DSetXRotationUsingRadians(matrix, degrees * Math.PI / 180.0);
		},
		m3DSetYRotationUsingRadians: function(radians) {
			var matrix = [];
			matrix[0] = Math.cos(radians);
			matrix[2] = Math.sin(radians);
			matrix[8] = -matrix[2];
			matrix[10] = matrix[0];
			matrix[1] = matrix[3] = matrix[4] = matrix[6] = matrix[7] = 0.0;
			matrix[9] = matrix[11] = matrix[13] = matrix[12] = matrix[14] = 0.0;
			matrix[5] = matrix[15] = 1.0;
			return matrix;
		},
		m3DSetYRotationUsingDegrees: function(degrees) {
			return obj.m3DSetYRotationUsingRadians(matrix, degrees * Math.PI / 180.0);
		},
		m3DSetZRotationUsingRadians: function(radians) {
			var matrix = [];
			matrix[0] = Math.cos(degrees);
			matrix[1] = Math.sin(degrees);
			matrix[4] = -matrix[1];
			matrix[5] = matrix[0];
			matrix[2] = matrix[3] = matrix[6] = matrix[7] = matrix[8] = 0.0;
			matrix[9] = matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;
			matrix[10] = matrix[15] = 1.0;
			return matrix;
		},
		m3DSetZRotationUsingDegrees: function(degrees) {
			return obj.m3DSetZRotationUsingRadians(degrees * M_PI / 180.0);
		},
		m3DSetRotationByRadians: function(angle, x, y, z) {
			matrix = [];
			var mag = Math.sqrt((x*x) + (y*y) + (z*z));
			if (mag == 0.0) {
				x = 1.0;
				y = 0.0;
				z = 0.0;
			} else if (mag != 1.0) {
				x /= mag;
				y /= mag;
				z /= mag;
			}
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			matrix[3] = matrix[7] = matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;
			matrix[15] = 1.0;

			matrix[0] = (x*x)*(1-c) + c;
			matrix[1] = (y*x)*(1-c) + (z*s);
			matrix[2] = (x*z)*(1-c) - (y*s);
			matrix[4] = (x*y)*(1-c)-(z*s);
			matrix[5] = (y*y)*(1-c)+c;
			matrix[6] = (y*z)*(1-c)+(x*s);
			matrix[8] = (x*z)*(1-c)+(y*s);
			matrix[9] = (y*z)*(1-c)-(x*s);
			matrix[10] = (z*z)*(1-c)+c;
			return matrix;
		},
		m3DSetRotationByDegrees: function(angle, x, y, z) {
			return obj.m3DSetRotationByRadians(angle * Math.PI / 180.0, x, y, z);
		},
		m3DSetShear: function(xShear, yShear) {
			var matrix = [];
			matrix[0] = matrix[5] =  matrix[10] = matrix[15] = 1.0;
			matrix[1] = matrix[2] = matrix[3] = 0.0;
			matrix[6] = matrix[7] = matrix[8] = matrix[9] = 0.0;
			matrix[11] = matrix[12] = matrix[13] = matrix[14] = 0.0;
			matrix[1] = xShear;
			matrix[4] = yShear;
			return matrix;
		},
		m3DInitMatrix: function() {
			var matrix = [];
			return obj.m3DSetIdentity(matrix);
		},
		m3DCopy : function(oldmat) {
			var newmat = [];
			for (var i = 0; i < 16; i++) {
				newmat[i] = oldmat[i];
			}
			return newmat;
		}
	};
	return obj;
})();
