/*  core_cubedemo.js
 *
 * Developed by:
 *   Richard M. Salter
 *   Logical Labs, LLC
 *
 * Copyright (c) 2011 by Logical Labs, LLC. All Rights Reserved.
 *
 */
 
var win;

exports.initialize = function(window) {
	win = window;
	var newDemo = Demo();	
	window.add(newDemo.opengl);
}

var Opengl = require('ti.opengl');

var Matrix = require('common/matrix').Matrix;
var GestureListener = require('common/gestures').GestureListener;
var Cube = require('core/lib/core_cubedata').Cube

var Demo = function() {
	var texFile = 'common/apll1.pvrtc';
	var opengl = Opengl.createView({
		backgroundColor:"#aaa",
		foregroundColor:"#fff",
		top:0,
		left:0,
		width:"100%",
		height:"100%",
		depthbuffer:true,
	});
	var tcallback =  function(deltaX, deltaY) {
		obj.drawView(obj.opengl, obj.scale, deltaX, deltaY);
	};
	var pcallback = function(scale) {
		obj.drawView(obj.opengl, scale, 0, 0);
	}
	var gestureListener = new GestureListener(opengl, pcallback, tcallback);
	var obj = {
		opengl : opengl,
		currentCalculatedMatrix : Matrix.m3DInitMatrix(),
		scale : 1,
		data : Cube(texFile),
		renderByRotatingAround : function(xRotation, yRotation) {
			var totalRotation = Math.sqrt(xRotation*xRotation + yRotation*yRotation);
			if (totalRotation == 0) {
				return;
			}
			var xt = xRotation/totalRotation;
			var yt = yRotation/totalRotation;
			var temporaryMatrix = Matrix.m3DSetRotationByDegrees(totalRotation,
			(xt * obj.currentCalculatedMatrix[1] + yt * obj.currentCalculatedMatrix[0]),
			(xt * obj.currentCalculatedMatrix[5] + yt * obj.currentCalculatedMatrix[4]),
			(xt * obj.currentCalculatedMatrix[9] + yt * obj.currentCalculatedMatrix[8]));
			var temporaryMatrix1 = Matrix.m3DMultiply(obj.currentCalculatedMatrix,temporaryMatrix);
			if ((temporaryMatrix1[0] >= -100.0) && (temporaryMatrix1[0] <= 100.0)) {
				 obj.currentCalculatedMatrix = temporaryMatrix1;
			}
		},
		configureLighting: function(view) {
			view.glEnable(Opengl.GL_BLEND);
			view.glEnable(Opengl.GL_LIGHTING);
			view.glEnable(Opengl.GL_LIGHT0);
			view.glBlendFunc(Opengl.GL_ONE, Opengl.GL_SRC_COLOR);
			var light0Ambient = [0.3, 0.3, 0.3, 1.0];
			view.glLightfv(Opengl.GL_LIGHT0, Opengl.GL_AMBIENT, light0Ambient);
			var light0Diffuse = [0.8, 0.8, 0.8, 1.0];
			view.glLightfv(Opengl.GL_LIGHT0, Opengl.GL_DIFFUSE, light0Diffuse);
			var light0Specular = [0.8, 0.8, 0.8, 1.0];
			view.glLightfv(Opengl.GL_LIGHT0, Opengl.GL_SPECULAR, light0Specular);
			light0Position = [0.0, 10.0, 10.0, 0.0];
			view.glLightfv(Opengl.GL_LIGHT0, Opengl.GL_POSITION, light0Position);
			var light0Direction = [0.0, 0.0, -1.0];
			view.glLightfv(Opengl.GL_LIGHT0, Opengl.GL_SPOT_DIRECTION, light0Direction);
			view.glLightf(Opengl.GL_LIGHT0, Opengl.GL_SPOT_CUTOFF, 90.0);
		},
		setup: function(view) {
			rect = view.bounds;
			var h = rect.height / rect.width;			
			view.glEnable(Opengl.GL_DEPTH_TEST);
			view.glMatrixMode(Opengl.GL_PROJECTION);
			view.glLoadIdentity();
			var zNear = .1;
			var zFar = 40.0;
			var fieldOfView = 45.0;
			var size = zNear * Math.tan((fieldOfView/2)*(Math.PI/180));
			view.glFrustumf({
				left:-size,
				right:size,
				bottom:(-size/(rect.width / rect.height)),
				top:(size/(rect.width/rect.height)),
				nearVal:zNear,
				farVal:zFar
			});
			view.glViewport(0, 0, rect.width, rect.height);
			view.glMatrixMode(Opengl.GL_MODELVIEW);
			view.glLoadIdentity();
			
			obj.configureLighting(view);
			
			view.glMaterialfv(Opengl.GL_FRONT_AND_BACK, Opengl.GL_AMBIENT, obj.data.matAmbient);
			view.glMaterialfv(Opengl.GL_FRONT_AND_BACK, Opengl.GL_DIFFUSE, obj.data.matDiffuse);
			view.glMaterialf(Opengl.GL_FRONT_AND_BACK, Opengl.GL_SHININESS, obj.data.matShininess);
			
			view.glEnable(Opengl.GL_CULL_FACE);
			view.glCullFace(Opengl.GL_FRONT);
			view.glEnable(Opengl.GL_TEXTURE_2D);
			texture = view.glGenTextures(1);
			view.glBindTexture(Opengl.GL_TEXTURE_2D, texture[0]);
			view.glTexParameteri(Opengl.GL_TEXTURE_2D, Opengl.GL_TEXTURE_MIN_FILTER, Opengl.GL_LINEAR);
			view.glTexParameteri(Opengl.GL_TEXTURE_2D, Opengl.GL_TEXTURE_MAG_FILTER, Opengl.GL_LINEAR);
			view.glTexParameterf(Opengl.GL_TEXTURE_2D, Opengl.GL_TEXTURE_MAX_ANISOTROPY_EXT, 1.0);
			view.glTexParameteri(Opengl.GL_TEXTURE_2D, Opengl.GL_TEXTURE_WRAP_S, Opengl.GL_CLAMP_TO_EDGE);
			view.glTexParameteri(Opengl.GL_TEXTURE_2D, Opengl.GL_TEXTURE_WRAP_T, Opengl.GL_CLAMP_TO_EDGE);
			view.glCompressedTexImage2D(Opengl.GL_TEXTURE_2D, 0, Opengl.GL_COMPRESSED_RGB_PVRTC_4BPPV1_IMG, 1024, 1024, 0, obj.data.tFile);
		},
		drawView : function(view, scale, deltaX, deltaY) {
			view.setFrameBuffer();
			view.clear();
			view.glLoadIdentity();
			view.glTranslatef(0.0, 0.0, -6.0);
			if (deltaX != null && deltaY != null) {
				obj.renderByRotatingAround(deltaX, -deltaY);
				view.glMultMatrixf(obj.currentCalculatedMatrix);
			}
			if (scale != null) {
				obj.scale = scale;
				view.glScalef(scale, scale,  scale);
			}
			
			obj.configureLighting(view);

			view.glEnableClientState(Opengl.GL_VERTEX_ARRAY);
			view.glEnableClientState(Opengl.GL_TEXTURE_COORD_ARRAY);
			view.glVertexPointer(3, Opengl.GL_FLOAT, 0, obj.data.cubeVertices);
			view.glTexCoordPointer(2, Opengl.GL_FLOAT, 0, obj.data.cubeTexCoords);
			for (var i = 0; i < 24; i += 4) {
				view.glDrawArrays(Opengl.GL_TRIANGLE_STRIP, i, 4);
			}
			view.glDisableClientState(Opengl.GL_VERTEX_ARRAY);
			view.glDisableClientState(Opengl.GL_TEXTURE_COORD_ARRAY);
			view.presentFrameBuffer();
		},
		start : function(e) {
			obj.setup(obj.opengl);
			obj.drawView(obj.opengl);			
		}
	};
	opengl.addEventListener('ready', obj.start);
	return obj;
};
