/*  core_geardemo.js 
 *
 * 3-D gear wheels.  This program is in the public domain.
 *
 * Brian Paul
 *
 * Conversion to OpenGL ES:  http://www.khronos.org/message_boards/viewtopic.php?f=4&t=1906
 * Conversion to Javascript for use with ti.opengl module by Richard M. Salter, Logical Labs
 * 
 * Developed for Appcelerator by:
 *   Richard M. Salter
 *   John V. Pataki
 *   Logical Labs, LLC
 * 
 * Copyright (c) 2011 by Appcelerator, Inc. All Rights Reserved.
 * 
 */

exports.initialize = function(window) {
	var newDemo = new GearDemo(window);
	TiOpenGLUtil.addComponents(window, newDemo.components);
	window.add(newDemo.components.opengl);
}

var OpenGL = require('ti.opengl');

var GearMaker = require('core/lib/core_gearMaker').GearMaker;
var Mover = require('common/mover').Mover;
var GestureListener = require('common/gestures').GestureListener;
var TiOpenGLUtil = require('common/helpers').TiOpenGLUtil;

var GearDemo = function(win) {
	var that = this;
	this.components = {
		opengl : OpenGL.createView({
			backgroundColor:"#aaa",
			top:0,
			left:0,
			width:"100%",
			height:"100%",
			depthbuffer:true,
		}),
		button : Ti.UI.createButton({
			style:Titanium.UI.iPhone.SystemButtonStyle.BORDERED,
			title:"Light Off",
			noadd:true,
  		}),
		slider: Ti.UI.createSlider({
			width: Ti.Platform.displayCaps.platformWidth / 2 - 20,
			min:0,
			max:250,
			value:100,
			title:"Speed",
			noadd:true				
		}),
		toolbar: Ti.UI.iOS.createToolbar({bottom:0}),
		flex: Titanium.UI.createButton({
              systemButton:Titanium.UI.iPhone.SystemButton.FLEXIBLE_SPACE,
              noadd:true
          	})
	};

	this.gears = function() {
		var red = [ 1.0, 0.1, 0.0, 1.0 ];
		var green = [ 0.0, 1.0, 0.2, 1.0 ];
		var blue = [ 0.2, 0.2, 1.0, 1.0 ];
		var ans =
			new Array(new GearMaker(1.0, 4.0, 1.0, 20, 0.7, red, OpenGL),
			new GearMaker(0.5, 2.0, 2.0, 10, 0.7, green, OpenGL),
			new GearMaker(1.3, 2.0, 0.5, 10, 0.7, blue, OpenGL));
		return ans;
	}();
	this.material = {
		specular : [0.73, 0.73, 0.73, 1.0],
		shininess : 80.0,
		emission : [0.1, 0.1, 0.1, 1.0]
	};
	this.vbufs = null;
	this.begin = function() {
		var opengl = that.components.opengl;
		var button = that.components.button;
		var slider = that.components.slider;
		var toolbar = that.components.toolbar;
		var speed = 100;
		var mover = new Mover(speed);
		var gestureListener = new GestureListener(opengl);
		var flex = that.components.flex;
		win.addEventListener('close', function(e) {
			clearInterval(that.timer);
		});

		toolbar.items = [flex,button,flex,slider,flex];
			slider.addEventListener('change', function(e) {
			mover.setSpeed(e.source.value)
		})
		button.addEventListener('click', function(e) {
			if (button.title == 'Light Off') {
				var light0Diffuse = [0.0, 0.0, 0.0, 1.0];
				opengl.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_DIFFUSE, light0Diffuse);
				button.title = 'Light On';
			} else {
				var light0Diffuse = [2.0, 2.0, 2.0, 1.0];
				opengl.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_DIFFUSE, light0Diffuse);
				button.title = 'Light Off';
			}
		});
		
		var view_rotx = 20.0, view_roty = 30.0, view_rotz = 0.0;
		var gear1, gear2, gear3;
		var angle = 0.0;
		var viewDist = 40.0;
		var transY = .5;
		var transZ = -40;
		var setCamera = function(view) {
			rect = view.bounds;
			var h = rect.height / rect.width;
			view.setCurrentContext();
			view.glMatrixMode(OpenGL.GL_PROJECTION);
			view.glLoadIdentity();
			view.glFrustumf(-1.0, 1.0, -h, h, 5.0, 200.0);
			view.glViewport(0, 0, rect.width, rect.height);
			view.glMatrixMode(OpenGL.GL_MODELVIEW);				
		}
		var setup = function(view) {
			setCamera(view);
				view.glEnable(OpenGL.GL_LIGHTING);
			view.glEnable(OpenGL.GL_LIGHT0);
			view.glEnable(OpenGL.GL_DEPTH_TEST);
			
			var pos = [ 10.0, 10.0, 10.0, 0 ];				
			view.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_POSITION, pos);
			var light0Ambient = [0.1, 0.1, 0.1, 1.0];
			view.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_AMBIENT, light0Ambient);
			var light0Diffuse = [1.0, 1.0, 1.0, 1.0];
			view.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_DIFFUSE, light0Diffuse);
			var light0Specular = [0.5, 0.5, 0.5, 1.0];
			view.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_SPECULAR, light0Specular);
			var light0Direction = [-1.0, 0.0, -1.0];
			view.glLightfv(OpenGL.GL_LIGHT0, OpenGL.GL_SPOT_DIRECTION, light0Direction);
			view.glLightf(OpenGL.GL_LIGHT0, OpenGL.GL_SPOT_CUTOFF, 100.0);
			view.glLightf(OpenGL.GL_LIGHT0, OpenGL.GL_SPOT_EXPONENT, 100.0);				
			view.glLightf(OpenGL.GL_LIGHT0, OpenGL.GL_CONSTANT_ATTENUATION, 1);								
			view.glLightf(OpenGL.GL_LIGHT0, OpenGL.GL_LINEAR_ATTENUATION, .5);												
			view.glLightf(OpenGL.GL_LIGHT0, OpenGL.GL_QUADRATIC_ATTENUATION, .25);																
				view.glEnable(OpenGL.GL_COLOR_MATERIAL);
			view.glMaterialfv(OpenGL.GL_FRONT_AND_BACK, OpenGL.GL_SPECULAR, that.material.specular);				
			view.glMaterialf(OpenGL.GL_FRONT_AND_BACK, OpenGL.GL_SHININESS, that.material.shininess);
			view.glMaterialfv(OpenGL.GL_FRONT_AND_BACK, OpenGL.GL_EMISSION, that.material.emission);								
				view.glEnableClientState(OpenGL.GL_NORMAL_ARRAY);
			view.glEnableClientState(OpenGL.GL_VERTEX_ARRAY);
				that.vbufs = that.gears.map( function(dbuf) {
				return TiOpenGLUtil.makeBuffers(view, dbuf);
			});
		}
		var drawImage = function(view, timestamp) {
			angle = mover.iterate(timestamp);
			if (angle > 3600.0)
				angle -= 3600.0;
			view.setFrameBuffer();
			view.clear();
			view.glPushMatrix();
			view.glTranslatef(1.0, transY, transZ);
			var scale = gestureListener.scale();
			view.glScalef(scale, scale, scale);
			view.glRotatef(view_rotx-100*gestureListener.ymove(), 1.0, 0.0, 0.0);
			view.glRotatef(view_roty+100*gestureListener.xmove(), 0.0, 1.0, 0.0);
			view.glRotatef(view_rotz, 0.0, 0.0, 1.0);
			view.glPushMatrix();
			view.glTranslatef(-3.0, -2.0, 0.0);
			view.glRotatef(angle, 0.0, 0.0, 1.0);
			draw_gear(view, 0);
			view.glPopMatrix();
			view.glPushMatrix();
			view.glTranslatef(3.1, -2.0, 0.0);
			view.glRotatef(-2.0 * angle - 9.0, 0.0, 0.0, 1.0);
			draw_gear(view, 1);
			view.glPopMatrix();
			view.glPushMatrix();
			view.glTranslatef(-3.1, 4.2, 0.0);
			view.glRotatef(-2.0 * angle - 25.0, 0.0, 0.0, 1.0);
			draw_gear(view, 2);
			view.glPopMatrix();
			view.glPopMatrix();
			view.presentFrameBuffer();
		}
		var draw_gear = function(view, idx) {
			var gear = that.gears[idx];
			var vbuf = that.vbufs[idx];
			
			view.glColor4f(gear.color);
			view.glMaterialfv(OpenGL.GL_FRONT_AND_BACK, OpenGL.GL_AMBIENT_AND_DIFFUSE, gear.color);
			view.glBindBuffer(OpenGL.GL_ARRAY_BUFFER, vbuf.vbo[0]);
			view.glBindBuffer(OpenGL.GL_ELEMENT_ARRAY_BUFFER, vbuf.vbo[1]);
			view.glVertexPointer(3, OpenGL.GL_FLOAT, 0, vbuf.start[0]);
			view.glNormalPointer(OpenGL.GL_FLOAT, 0, vbuf.start[1]);
			view.glDrawElements(OpenGL.GL_TRIANGLES, gear.nindices/3, OpenGL.GL_UNSIGNED_SHORT, 0);
		}
		var init = function() {
			var interval = 33.33333;
			setup(opengl);
			TiOpenGLUtil.startAnimation(opengl, drawImage, interval, 100, this);				
		}
		init();
	};

	this.components.opengl.addEventListener('ready', this.begin);
};
