/*  core_gearMaker.js
 *
 * 3-D gear wheels.  This program is in the public domain.
 *
 * Brian Paul
 *
 * Conversion to OpenGL ES:  http://www.khronos.org/message_boards/viewtopic.php?f=4&t=1906
 * Conversion to Javascript for use with ti.opengl module by Richard M. Salter, Logical Labs
 * 
 * Developed for Appcelerator by:
 *   Richard M. Salter
 *   Logical Labs, LLC
 * 
 * Copyright (c) 2011 by Appcelerator, Inc. All Rights Reserved.
 * 
 */

exports.GearMaker = function(inner_radius, outer_radius, width, teeth, tooth_depth, color_, openGL) {
	function push3(vx, x, y, z) {
		vx.push(x);
		vx.push(y);
		vx.push(z);
	}

	var vertices = new Array();
	var normals = new Array();
	var indices = new Array();
	this.nindices = teeth*66*3
	this.color = color_

	var i, j
	var r0, r1, r2;
	var ta, da;
	var u1, v1, u2, v2, len;
	var cos_ta, cos_ta_1da, cos_ta_2da, cos_ta_3da, cos_ta_4da;
	var sin_ta, sin_ta_1da, sin_ta_2da, sin_ta_3da, sin_ta_4da;
	var ix0, ix1, ix2, ix3, ix4, ix5;
	var vx, nx, ix;
	var ctr = 0;
	var ictr = 0;

	r0 = inner_radius;
	r1 = outer_radius - tooth_depth / 2.0;
	r2 = outer_radius + tooth_depth / 2.0;
	da = (2.0 * Math.PI / teeth) / 4.0;

	vx = vertices;
	nx = normals;
	ix = indices;

	for (i = 0; i < teeth; i++) {
		ta = i * 2.0 * Math.PI / teeth;

		cos_ta = Math.cos(ta);
		cos_ta_1da = Math.cos(ta + da);
		cos_ta_2da = Math.cos(ta + 2 * da);
		cos_ta_3da = Math.cos(ta + 3 * da);
		cos_ta_4da = Math.cos(ta + 4 * da);
		sin_ta = Math.sin(ta);
		sin_ta_1da = Math.sin(ta + da);
		sin_ta_2da = Math.sin(ta + 2 * da);
		sin_ta_3da = Math.sin(ta + 3 * da);
		sin_ta_4da = Math.sin(ta + 4 * da);

		u1 = r2 * cos_ta_1da - r1 * cos_ta;
		v1 = r2 * sin_ta_1da - r1 * sin_ta;
		len = Math.sqrt(u1 * u1 + v1 * v1);
		u1 /= len;
		v1 /= len;
		u2 = r1 * cos_ta_3da - r2 * cos_ta_2da;
		v2 = r1 * sin_ta_3da - r2 * sin_ta_2da;

		/* front face */

		push3(vx, r0 * cos_ta,          r0 * sin_ta,          width * 0.5);
		push3(nx, 0, 0, 1);
		ix0 = ctr++;
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix1 = ctr++;
		push3(vx, r0 * cos_ta,          r0 * sin_ta,          width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix2 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix3 = ctr++;
		push3(vx, r0 * cos_ta_4da,      r0 * sin_ta_4da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix4 = ctr++;
		push3(vx, r1 * cos_ta_4da,      r1 * sin_ta_4da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix5 = ctr++;

		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;
		ix[ictr++] = ix2, ix[ictr++] = ix3;
		ix[ictr++] = ix4;
		ix[ictr++] = ix3, ix[ictr++] = ix5;
		ix[ictr++] = ix4;

		/* front sides of teeth */
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix0 = ctr++;
		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix1 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix2 = ctr++
		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      width * 0.5);
		push3(nx,  0, 0, 1.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		/* back face */
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix0 = ctr++;
		push3(vx, r0 * cos_ta,          r0 * sin_ta,          -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix1 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix2 = ctr++;
		push3(vx, r0 * cos_ta,          r0 * sin_ta,          -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix3 = ctr++;
		push3(vx, r1 * cos_ta_4da,      r1 * sin_ta_4da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix4 = ctr++;
		push3(vx, r0 * cos_ta_4da,      r0 * sin_ta_4da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix5 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;
		ix[ictr++] = ix2, ix[ictr++] = ix3;
		ix[ictr++] = ix4;
		ix[ictr++] = ix3, ix[ictr++] = ix5;
		ix[ictr++] = ix4;

		/* back sides of teeth */
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix0 = ctr++;
		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix1 = ctr++;
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix2 = ctr++;
		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      -width * 0.5);
		push3(nx,  0, 0, -1.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		/* draw outward faces of teeth */
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          width * 0.5);
		push3(nx,  v1, -u1, 0);
		ix0 = ctr++;
		push3(vx, r1 * cos_ta,          r1 * sin_ta,          -width * 0.5);
		push3(nx,  v1, -u1, 0);
		ix1 = ctr++;
		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      width * 0.5);
		push3(nx,  v1, -u1, 0);
		ix2 = ctr++;
		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      -width * 0.5);
		push3(nx,  v1, -u1, 0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix0 = ctr++;
		push3(vx, r2 * cos_ta_1da,      r2 * sin_ta_1da,      -width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix1 = ctr++;
		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix2 = ctr++;
		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      -width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      width * 0.5);
		push3(nx,  v2, -u2, 0.0);
		ix0 = ctr++;
		push3(vx, r2 * cos_ta_2da,      r2 * sin_ta_2da,      -width * 0.5);
		push3(nx,  v2, -u2, 0.0);
		ix1 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      width * 0.5);
		push3(nx,  v2, -u2, 0.0);
		ix2 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      -width * 0.5);
		push3(nx,  v2, -u2, 0.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix0 = ctr++;
		push3(vx, r1 * cos_ta_3da,      r1 * sin_ta_3da,      -width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix1 = ctr++;
		push3(vx, r1 * cos_ta_4da,      r1 * sin_ta_4da,      width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix2 = ctr++;
		push3(vx, r1 * cos_ta_4da,      r1 * sin_ta_4da,      -width * 0.5);
		push3(nx,  cos_ta, sin_ta, 0.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;

		/* draw inside radius cylinder */
		push3(vx, r0 * cos_ta,          r0 * sin_ta,          -width * 0.5);
		push3(nx,  -cos_ta, -sin_ta, 0.0);
		ix0 = ctr++;
		push3(vx, r0 * cos_ta,          r0 * sin_ta,          width * 0.5);
		push3(nx,  -cos_ta, -sin_ta, 0.0);
		ix1 = ctr++;
		push3(vx, r0 * cos_ta_4da,      r0 * sin_ta_4da,      -width * 0.5);
		push3(nx,  -cos_ta_4da, -sin_ta_4da, 0.0);
		ix2 = ctr++;
		push3(vx, r0 * cos_ta_4da,      r0 * sin_ta_4da,      width * 0.5);
		push3(nx,  -cos_ta_4da, -sin_ta_4da, 0.0);
		ix3 = ctr++;
		ix[ictr++] = ix0, ix[ictr++] = ix1;
		ix[ictr++] = ix2;
		ix[ictr++] = ix1, ix[ictr++] = ix3;
		ix[ictr++] = ix2;
	}
    this.vertexDB = openGL.createDataBuffer({
    	data: vertices,
    	type: openGL.GL_FLOAT
    });
    this.normalDB = openGL.createDataBuffer({
    	data: normals,
    	type: openGL.GL_FLOAT
    });
    this.indexDB = openGL.createDataBuffer({
    	data: indices,
    	type: openGL.GL_UNSIGNED_SHORT
    });
}
