/*  enhanced_cubedemo.js
 *
 * Developed by:
 *   Richard M. Salter
 *   Logical Labs, LLC
 *
 * Copyright (c) 2011 by Logical Labs, LLC. All Rights Reserved.
 *
 */

exports.initialize = function(window) {
	var newDemo = new CubeDemo();
	window.add(newDemo.opengl);
}

var OpenGL = require('ti.opengl');

var Matrix = require('common/matrix').Matrix;
var GestureListener = require('common/gestures').GestureListener;
var Cube = require('enhanced/lib/enhanced_cubedata').Cube

var CubeDemo = function() {
	var that = this;
	var texFile = 'common/apll1.pvrtc';
	this.opengl = OpenGL.createView({
		backgroundColor : "#aaa",
		top : 0,
		left : 0,
		width : "100%",
		height : "100%",
		depthbuffer : true,
	});
	var camera0 = OpenGL.createCamera({
		zNear : 0.1,
		zFar : 40.0,
		fieldOfView : 45.0,
		position : {
			x : 0.0,
			y : 0.0,
			z : 6.0
		},
		angle : {
			x : 0.0,
			y : 0.0,
			z : 0.0
		}
	});
	var graphics = OpenGL.createGraphics3D({
		view : this.opengl,
		camera : camera0,
		lights : [{
			ambient : {
				r : 0.3,
				g : 0.3,
				b : 0.3,
				a : 1
			},
			diffuse : {
				r : 0.8,
				g : 0.8,
				b : 0.8,
				a : 1
			},
			specular : {
				r : 0.8,
				g : 0.8,
				b : 0.8,
				a : 1
			},
			position : {
				x : 0.0,
				y : 10.0,
				z : 10.0,
				w : 0
			},
			spotDirection : {
				x : 0.0,
				y : 0.0,
				z : -1.0
			},
			spotCuttoff : 90,
		}],
		operations : {
			glEnable_0 : [OpenGL.GL_BLEND],
			glBlendFunc : [OpenGL.GL_ONE, OpenGL.GL_SRC_COLOR],
			glEnable_1 : [OpenGL.GL_CULL_FACE],
			glCullFace : [OpenGL.GL_FRONT],
			glEnable_2 : [OpenGL.GL_TEXTURE_2D]
		}
	});
	var tcallback = function(deltaX, deltaY) {
		that.renderByRotatingAround(deltaX, -deltaY);
		that.drawView();
	};
	var pcallback = function(scale) {
		that.scaleRatio = scale / that.scale;
		that.scale = scale;
		that.buildScaleMat(that.currentCalculatedMatrix);
		that.drawView();
	};
	var gestureListener = new GestureListener(this.opengl, pcallback, tcallback);
	this.graphics = graphics;
	this.currentCalculatedMatrix = Matrix.m3DInitMatrix();
	this.scale = 1;
	this.scaleRatio = 1;
	this.cube = Cube(4, {
		x : 0,
		y : 20,
		z : 0
	}, texFile, this.opengl,OpenGL);
	this.renderByRotatingAround = function(xRotation, yRotation) {
		var totalRotation = Math.sqrt(xRotation * xRotation + yRotation * yRotation);
		if(totalRotation == 0) {
			return;
		}
		var xt = xRotation / totalRotation;
		var yt = yRotation / totalRotation;
		var temporaryMatrix = Matrix.m3DSetRotationByDegrees(totalRotation, (xt * this.currentCalculatedMatrix[1] + yt * this.currentCalculatedMatrix[0]), (xt * this.currentCalculatedMatrix[5] + yt * this.currentCalculatedMatrix[4]), (xt * this.currentCalculatedMatrix[9] + yt * this.currentCalculatedMatrix[8]));
		var temporaryMatrix1 = Matrix.m3DMultiply(this.currentCalculatedMatrix, temporaryMatrix);
		if((temporaryMatrix1[0] >= -100.0) && (temporaryMatrix1[0] <= 100.0)) {
			this.buildScaleMat(temporaryMatrix1);
		}
	};
	this.buildScaleMat = function(mat) {
		var scaleRatio = this.scaleRatio;
		var sizemat = Matrix.m3DSetScaling(scaleRatio, scaleRatio, scaleRatio);
		var temporaryMatrix2 = Matrix.m3DMultiply(mat, sizemat);
		this.currentCalculatedMatrix = temporaryMatrix2;
		this.cube.matrix = this.currentCalculatedMatrix;
	};
	this.drawView = function() {
		graphics.begin();
		graphics.render(this.cube);
		graphics.end();
	};
	this.start = function(e) {
		setTimeout(function() {
			that.drawView();
		}, 100);
	};
	this.cube.matrix = this.currentCalculatedMatrix;
	this.opengl.addEventListener('ready', this.start);
};
