/*  enhanced_gearsdemo.js
 *
 * 3-D gear wheels.  This program is in the public domain.
 *
 * Brian Paul
 *
 * Conversion to OpenGL ES:  http://www.khronos.org/message_boards/viewtopic.php?f=4&t=1906
 * Conversion to Javascript for use with ti.opengl module by Richard M. Salter, Logical Labs
 *
 * Developed for Appcelerator by:
 *   Richard M. Salter
 *   John V. Pataki
 *   Logical Labs, LLC
 *
 * Copyright (c) 2011 by Appcelerator, Inc. All Rights Reserved.
 *
 */

exports.initialize = function(window) {
	var newDemo = new GearDemo(window);
	TiOpenGLUtil.addComponents(window, newDemo.components);
	window.add(newDemo.components.opengl);
}

var OpenGL = require('ti.opengl');

var GearMaker = require('enhanced/lib/enhanced_gearMaker').GearMaker;
var Mover = require('common/mover').Mover;
var GestureListener = require('common/gestures').GestureListener;
var TiOpenGLUtil = require('common/helpers').TiOpenGLUtil;

var GearDemo = function(win) {
	var that = this;
	var ops = {
		glEnable_0 : [OpenGL.GL_CULL_FACE],
		glEnable_1 : [OpenGL.GL_NORMALIZE],
		glEnable_2 : [OpenGL.GL_COLOR_MATERIAL],
		glShadeModel : [OpenGL.GL_SMOOTH],
	};
	var camera = OpenGL.createCamera({
		zNear : 5,
		zFar : 200,
		fieldOfView : 45.0,
		position : {
			x : -5.0,
			y : -.5,
			z : 40.0
		}
	});
	var opengl = OpenGL.createView({
		backgroundColor : "#aaa",
		top : 0,
		left : 0,
		width : "100%",
		height : "100%",
		depthbuffer : true,
	});
	var lights =
		[
			[{
				position: {x: 10, y : 10.0, z : 10.0, w: 0},
				ambient : {r : 0.1, g : 0.1, b : 0.1, a : 1.0},
				diffuse : {r : 0.0, g : 0.0, b : 0.0, a : 1.0},
				specular : {r : 0.5, g : 0.5, b : 0.5, a : 1.0},
				spotDirection : {x : -1.0, y : 0.0, z : -1.0},
				spotCutoff : 100.0,
				spotExponent : 100.0,
				constantAttenuation : 1,
				linearAttenuation : .5,
				quadAttenuation : .25,
			}],
			[{
				position: {x: 10, y : 10.0, z : 10.0, w: 0},
				ambient : {r : 0.1, g : 0.1, b : 0.1, a : 1.0},
				diffuse : {r : 2.0, g : 2.0, b : 2.0, a : 1.0},
				specular : {r : 0.5, g : 0.5, b : 0.5, a : 1.0},
				spotDirection : {x : -1.0, y : 0.0, z : -1.0},
				spotCutoff : 100.0,
				spotExponent : 100.0,
				constantAttenuation : 1,
				linearAttenuation : .5,
				quadAttenuation : .25,
			}],
			
	];
	var graphics = OpenGL.createGraphics3D({
		view : opengl,
		camera : camera,
		operations : ops,
	        lights : lights[1]
	});
	var view_rotx = -20.0, view_roty = -30.0, view_rotz = 0.0;
	var speed = 100;
	var mover = new Mover(speed);
	var gears = function() {
		var red = [1.0, 0.1, 0.0, 1.0];
		var green = [0.0, 1.0, 0.2, 1.0];
		var blue = [0.2, 0.2, 1.0, 1.0];
		var loc0 = {
			x : -3.0,
			y : -2.0,
			z : 0.0
		};
		var loc1 = {
			x : 3.1,
			y : -2.0,
			z : 0.0
		};
		var loc2 = {
			x : -3.1,
			y : 4.2,
			z : 0.0
		}
		var ans = new Array(new GearMaker(1.0, 4.0, 1.0, 20, 0.7, red, 1, loc0, OpenGL), new GearMaker(0.5, 2.0, 2.0, 10, 0.7, green, 1, loc1, OpenGL), new GearMaker(1.3, 2.0, 0.5, 10, 0.7, blue, 1, loc2, OpenGL));
		return ans;
	}();
	var gearset = OpenGL.createScene({
		models : gears,
		angle : {
			x : view_rotx,
			y : view_roty,
			z : view_rotz
		},
		size : {
			x : 2,
			y : 2,
			z : 2
		},
		material : {
			specular : "#bababa",
			shininess : 80,
			emission : [0.1, 0.1, 0.1, 1.0]
		}
	});
	this.opengl = opengl;
	this.graphics = graphics;
	this.gestureListener = null;
	this.animator = OpenGL.createAnimator({
		view : opengl,
        synchronous : true,
		callback : function(e) {
			that.drawImage(e.view, e.timestamp);
		}
	});
	this.components = {
		opengl : opengl,
		button : Ti.UI.createButton({
			style : Titanium.UI.iPhone.SystemButtonStyle.BORDERED,
			title : "Light Off",
			noadd : true
		}),
		slider : Ti.UI.createSlider({
			width: Ti.Platform.displayCaps.platformWidth / 2 - 20,
			min : 0,
			max : 250,
			value : 100,
			title : "Speed",
			noadd : true
		}),
		toolbar : Ti.UI.iOS.createToolbar({
			bottom : 0
		}),
		flex : Titanium.UI.createButton({
			systemButton : Titanium.UI.iPhone.SystemButton.FLEXIBLE_SPACE,
			noadd : true
		})
	};
	this.drawImage = function(view, timestamp) {
		theta = 1000 * mover.iterate(timestamp);
		if(theta > 3600.0)
			theta -= 3600.0;
		gears[0].vRotation = {
			angle : theta,
			x : 0,
			y : 0,
			z : 1
		};
		gears[1].vRotation = {
			angle : -2.0 * theta - 9.0,
			x : 0,
			y : 0,
			z : 1
		};
		gears[2].vRotation = {
			angle : -2.0 * theta - 25.0,
			x : 0,
			y : 0,
			z : 1
		};
		graphics.begin();
		graphics.render(gearset);
		graphics.end();
	};
	this.pcallback = function(nscale) {
		gearset.size = {
			x : nscale,
			y : nscale,
			z : nscale
		}
	};
	this.tcallback = function(deltaX, deltaY) {
		gearset.angle = {
			x : gearset.angle.x + deltaY,
			y : gearset.angle.y - deltaX,
			z : gearset.angle.z
		}
	};
	this.rcallback = function(nrot) {
		var rot = 20 * nrot
		camera.roll(-rot);
	};
	this.begin = function() {
		var opengl = that.components.opengl;
		var button = that.components.button;
		var slider = that.components.slider;
		var toolbar = that.components.toolbar;
		var flex = that.components.flex;
		win.addEventListener('close', function(e) {
			that.animator.stopAnimation();
		}); 
		that.gestureListener = new GestureListener(that.opengl, that.pcallback, that.tcallback, that.rcallback); 
		toolbar.items = [flex,button,flex,slider,flex];
		slider.addEventListener('change', function(e) {
			mover.setSpeed(e.source.value)
		})
		button.addEventListener('click', function(e) {
			if(button.title == 'Light Off') {
			        graphics.lights = lights[0];
				button.title = 'Light On';
			} else {
			        graphics.lights = lights[1];				
				button.title = 'Light Off';
			}
		});
		that.drawImage(opengl, 0);
		that.animator.startAnimation();
	};
	this.components.opengl.addEventListener('ready', this.begin);
};
