/*  enhanced_cubedata.js
 *
 * Developed by:
 *   Richard M. Salter
 *   Logical Labs, LLC
 *
 * Copyright (c) 2011 by Logical Labs, LLC. All Rights Reserved.
 *
 */

exports.Cube = function(size, location, tfile, view,OpenGL) {
	view.setCurrentContext();
	var f = Titanium.Filesystem.getFile(Titanium.Filesystem.resourcesDirectory, tfile);
	var blob = f.read();
	view.glEnable(OpenGL.GL_TEXTURE_2D);
	var texture = OpenGL.createTexture({
		filter : OpenGL.GL_LINEAR,
		view : view,
		compressedImage : blob,
		width : 1024,
		height : 1024,
		parameters: [
			{pname: OpenGL.GL_TEXTURE_MAX_ANISOTROPY_EXT, fvalue: 1.0},
			{pname: OpenGL.GL_TEXTURE_WRAP_S, ivalue: OpenGL.GL_CLAMP_TO_EDGE},
			{pname: OpenGL.GL_TEXTURE_WRAP_T, ivalue: OpenGL.GL_CLAMP_TO_EDGE}
		]		
	});
	var data = {
			cubeVertices : [
			// FRONT
			-1.0, -1.0,  -1.0,
			1.0, -1.0,  -1.0,
			-1.0,  1.0,  -1.0,
			1.0,  1.0,  -1.0,
			// BACK
			-1.0, -1.0, 1.0,
			-1.0,  1.0, 1.0,
			1.0, -1.0, 1.0,
			1.0,  1.0, 1.0,
			// LEFT
			-1.0, -1.0,  -1.0,
			-1.0,  1.0,  -1.0,
			-1.0, -1.0, 1.0,
			-1.0,  1.0, 1.0,
			// RIGHT
			1.0, -1.0, 1.0,
			1.0,  1.0, 1.0,
			1.0, -1.0, -1.0,
			1.0,  1.0, -1.0,
			// TOP
			-1.0, 1.0, -1.0,
			1.0,  1.0,  -1.0,
			-1.0, 1.0, 1.0,
			1.0,  1.0, 1.0,
			// BOTTOM
			-1.0, -1.0,  -1.0,
			-1.0, -1.0, 1.0,
			1.0, -1.0,  -1.0,
			1.0, -1.0, 1.0,
			],
			cubeTexCoords :
			[//front
			0.0, 0.0,
			0.5, 0.0,
			0.0, 0.5,
			0.5, 0.5,
			//back
			0.5, 0.0,
			0.5, 0.5,
			0.0, 0.0,
			0.0, 0.5,
			//left
			0.5, 0.5,
			0.5, 1.0,
			0.0, 0.5,
			0.0, 1.0,
			//right
			0.5, 0.5,
			0.5, 1.0,
			0.0, 0.5,
			0.0, 1.0,
			//top
			0.0, 0.0,
			1.0, 0.0,
			0.0, 1.0,
			1.0, 1.0,
			//bottom
			1.0, 0.0,
			1.0, 1.0,
			0.0, 0.0,
			0.0, 1.0
			],
	};
	var model = OpenGL.createModel({
        reLight: true,
		mode:OpenGL.GL_TRIANGLE_STRIP,
		vertices:data.cubeVertices,
		vSize:3,
		texCoords:data.cubeTexCoords,
		tSize:2,
		size:{x:size, y:size, z:size},
		position:location,
		texture:texture,
		grouping : 4,
		materials : {
			ambient: [1.0, 1.0, 1.0, 1.0], 
			diffuse: [1.0, 1.0, 1.0, 1.0],
			shininess : 100 
		}
	});
	return model;	
}
